@@ -15,10 +15,11 @@ from django.core.exceptions import ValidationError
 from django.test import TestCase
 from django.test.utils import override_settings
 from django.urls import reverse
+from django.utils import timezone
 
 from model_mommy import mommy
 
-from admin_tools_stats.models import DashboardStats, DashboardStatsCriteria
+from admin_tools_stats.models import DashboardStatsCriteria
 from admin_tools_stats.utils import BaseAuthenticatedClient, assertContainsAny
 
 
@@ -73,6 +74,37 @@ class AdminToolsStatsAdminCharts(BaseAuthenticatedClient):
             html=True,
         )
 
+    def test_admin_dashboard_page_multi_series(self):
+        stats = mommy.make(
+            'DashboardStats',
+            date_field_name="date_joined",
+            model_name="User",
+            model_app_name="auth",
+            graph_key="user_graph1",
+        )
+        criteria = mommy.make(
+            'DashboardStatsCriteria',
+            criteria_name="active",
+            dynamic_criteria_field_name="is_active",
+            criteria_dynamic_mapping={
+                "": [None, "All"],
+                "false": [False, "Inactive"],
+                "true": [True, "Active"],
+            },
+        )
+        mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=stats, use_as='multiple_series')
+        response = self.client.get('/admin/')
+        self.assertContains(
+            response,
+            '<option class="chart-input" value="">-------</option>',
+            html=True,
+        )
+        self.assertContains(
+            response,
+            '<option class="chart-input" value="%s" selected="selected">active</option>' % stats.id,
+            html=True,
+        )
+
     def test_admin_dashboard_page_post(self):
         """Test function to check dashboardstatscriteria admin pages"""
         response = self.client.post('/admin/', {'select_box_user_graph': 'true'})
@@ -99,6 +131,13 @@ class ModelTests(TestCase):
             model_app_name="auth",
             graph_key="user_graph",
         )
+        self.kid_stats = mommy.make(
+            'DashboardStats',
+            date_field_name="birthday",
+            model_name="TestKid",
+            model_app_name="demoproject",
+            graph_key="kid_graph",
+        )
 
     def test_clean_error_model_app_app_name(self):
         stats = mommy.make('DashboardStats', model_name="User1", model_app_name="auth1", graph_key="error_graph")
@@ -124,8 +163,8 @@ class ModelTests(TestCase):
     def test_get_multi_series(self):
         """Test function to check DashboardStats.get_multi_time_series()"""
         mommy.make('User', date_joined=datetime.date(2010, 10, 10))
-        time_since = datetime.date(2010, 10, 8)
-        time_until = datetime.date(2010, 10, 12)
+        time_since = datetime.datetime(2010, 10, 8)
+        time_until = datetime.datetime(2010, 10, 12)
 
         interval = "days"
         serie = self.stats.get_multi_time_series({}, time_since, time_until, interval)
@@ -138,6 +177,56 @@ class ModelTests(TestCase):
         }
         self.assertDictEqual(serie, testing_data)
 
+    @override_settings(USE_TZ=True, TIME_ZONE='Europe/Prague')
+    def test_get_multi_series_datetime_tz(self):
+        """Test function to check DashboardStats.get_multi_time_series()"""
+        current_tz = timezone.get_current_timezone()
+        mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=current_tz))
+        time_since = datetime.datetime(2010, 10, 9, 0, 0)
+        time_until = datetime.datetime(2010, 10, 11, 0, 0)
+
+        interval = "days"
+        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval)
+        testing_data = {
+            current_tz.localize(datetime.datetime(2010, 10, 9, 0, 0)): {'': 0},
+            current_tz.localize(datetime.datetime(2010, 10, 10, 0, 0)): {'': 1},
+            current_tz.localize(datetime.datetime(2010, 10, 11, 0, 0)): {'': 0},
+        }
+        self.assertDictEqual(serie, testing_data)
+
+    @override_settings(USE_TZ=True, TIME_ZONE='CET')
+    def test_get_multi_series_date_tz(self):
+        """Test function to check DashboardStats.get_multi_time_series()"""
+        mommy.make('TestKid', birthday=datetime.date(2010, 10, 10))
+        time_since = datetime.datetime(2010, 10, 9)
+        time_until = datetime.datetime(2010, 10, 11)
+
+        interval = "days"
+        serie = self.kid_stats.get_multi_time_series({}, time_since, time_until, interval)
+        testing_data = {
+            datetime.date(2010, 10, 9): {'': 0},
+            datetime.date(2010, 10, 10): {'': 1},
+            datetime.date(2010, 10, 11): {'': 0},
+        }
+        self.assertDictEqual(serie, testing_data)
+
+    @override_settings(USE_TZ=True, TIME_ZONE='Europe/Prague')
+    def test_get_multi_series_change_dst(self):
+        """Test function to check DashboardStats.get_multi_time_series() on edge of daylight saving time change """
+        current_tz = timezone.get_current_timezone()
+        mommy.make('User', date_joined=datetime.datetime(2019, 10, 28, tzinfo=current_tz))
+        time_since = datetime.datetime(2019, 10, 27, 0, 0)
+        time_until = datetime.datetime(2019, 10, 29, 0, 0)
+
+        interval = "days"
+        serie = self.stats.get_multi_time_series({}, time_since, time_until, interval)
+        testing_data = {
+            current_tz.localize(datetime.datetime(2019, 10, 27, 0, 0)): {'': 0},
+            current_tz.localize(datetime.datetime(2019, 10, 28, 0, 0)): {'': 1},
+            current_tz.localize(datetime.datetime(2019, 10, 29, 0, 0)): {'': 0},
+        }
+        self.assertDictEqual(serie, testing_data)
+
     @override_settings(USE_TZ=False)
     def test_get_multi_series_distinct_count(self):
         """Test function to check DashboardStats.get_multi_time_series() with distinct count."""
@@ -153,8 +242,8 @@ class ModelTests(TestCase):
         mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Foo")
         mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Foo")
         mommy.make('User', date_joined=datetime.date(2010, 10, 10), first_name="Bar")
-        time_since = datetime.date(2010, 10, 8)
-        time_until = datetime.date(2010, 10, 12)
+        time_since = datetime.datetime(2010, 10, 8)
+        time_until = datetime.datetime(2010, 10, 12)
 
         interval = "days"
         serie = stats.get_multi_time_series({}, time_since, time_until, interval)
@@ -190,12 +279,12 @@ class ModelTests(TestCase):
             mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=12)
             mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=1)
             mommy.make('TestKid', birthday=datetime.date(2010, 10, 10), age=2)
-            time_since = datetime.date(2010, 10, 9)
-            time_until = datetime.date(2010, 10, 10)
+            time_since = datetime.datetime(2010, 10, 9)
+            time_until = datetime.datetime(2010, 10, 10)
 
             interval = "days"
             serie = stats.get_multi_time_series({}, time_since, time_until, interval)
-            self.assertEqual(serie[datetime.datetime(2010, 10, 10, 0, 0)][''], result, "Bad value for function %s" % func),
+            self.assertEqual(serie[datetime.date(2010, 10, 10)][''], result, "Bad value for function %s" % func)
 
     @override_settings(USE_TZ=False)
     def test_get_multi_series_dynamic_field_name(self):
@@ -209,17 +298,15 @@ class ModelTests(TestCase):
                 "false": [False, "Inactive"],
                 "true": [True, "Active"],
             },
-            use_as='multiple_series',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        serie = self.stats.get_multi_time_series({'select_box_multiple_series': criteria.id}, time_since, time_until, interval)
+        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
             datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
@@ -243,17 +330,15 @@ class ModelTests(TestCase):
                 "False": "Inactive",
                 "True": "Active",
             },
-            use_as='multiple_series',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        serie = self.stats.get_multi_time_series({'select_box_multiple_series': criteria.id}, time_since, time_until, interval)
+        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
             datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Active', 0), ('Inactive', 0))),
@@ -273,17 +358,15 @@ class ModelTests(TestCase):
             'DashboardStatsCriteria',
             criteria_name="active",
             dynamic_criteria_field_name="is_active",
-            use_as='multiple_series',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), is_active=True)
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), is_active=False)
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        serie = self.stats.get_multi_time_series({'select_box_multiple_series': criteria.id}, time_since, time_until, interval)
+        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('True', 0), ('False', 0))),
             datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('True', 0), ('False', 0))),
@@ -303,18 +386,16 @@ class ModelTests(TestCase):
             'DashboardStatsCriteria',
             criteria_name="name",
             dynamic_criteria_field_name="last_name",
-            use_as='multiple_series',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo")
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar")
         mommy.make('User', date_joined=datetime.date(2010, 10, 14))
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        serie = self.stats.get_multi_time_series({'select_box_multiple_series': criteria.id}, time_since, time_until, interval)
+        serie = self.stats.get_multi_time_series({'select_box_multiple_series': m2m.id}, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
             datetime.datetime(2010, 10, 11, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
@@ -334,24 +415,21 @@ class ModelTests(TestCase):
             'DashboardStatsCriteria',
             criteria_name="name",
             dynamic_criteria_field_name="last_name",
-            use_as='multiple_series',
         )
         criteria_active = mommy.make(
             'DashboardStatsCriteria',
             criteria_name="active",
             dynamic_criteria_field_name="is_active",
-            use_as='chart_filter',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.criteria.add(criteria_active)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
+        m2m_active = mommy.make('CriteriaToStatsM2M', criteria=criteria_active, stats=self.stats, use_as='chart_filter')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo", is_active=True)
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar", is_active=False)
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        arguments = {'select_box_multiple_series': criteria.id, 'select_box_dynamic_%s' % criteria_active.id: 'True'}
+        arguments = {'select_box_multiple_series': m2m.id, 'select_box_dynamic_%s' % m2m_active.id: 'True'}
         serie = self.stats.get_multi_time_series(arguments, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
@@ -372,24 +450,21 @@ class ModelTests(TestCase):
             'DashboardStatsCriteria',
             criteria_name="name",
             dynamic_criteria_field_name="last_name",
-            use_as='multiple_series',
         )
         criteria_active = mommy.make(
             'DashboardStatsCriteria',
             criteria_name="active",
             criteria_fix_mapping={"is_active": True},
-            use_as='chart_filter',
         )
-        self.stats.criteria.add(criteria)
-        self.stats.criteria.add(criteria_active)
-        self.stats.save()
+        m2m = mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series')
+        mommy.make('CriteriaToStatsM2M', criteria=criteria_active, stats=self.stats, use_as='chart_filter')
         mommy.make('User', date_joined=datetime.date(2010, 10, 12), last_name="Foo", is_active=True)
         mommy.make('User', date_joined=datetime.date(2010, 10, 13), last_name="Bar", is_active=False)
-        time_since = datetime.date(2010, 10, 10)
-        time_until = datetime.date(2010, 10, 14)
+        time_since = datetime.datetime(2010, 10, 10)
+        time_until = datetime.datetime(2010, 10, 14)
 
         interval = "days"
-        arguments = {'select_box_multiple_series': criteria.id}
+        arguments = {'select_box_multiple_series': m2m.id}
         serie = self.stats.get_multi_time_series(arguments, time_since, time_until, interval)
         testing_data = {
             datetime.datetime(2010, 10, 10, 0, 0): OrderedDict((('Bar', 0), ('Foo', 0))),
@@ -433,12 +508,8 @@ class ViewsTests(BaseAuthenticatedClient):
                 "false": [True, "Inactive"],
                 "true": [False, "Active"],
             },
-            use_as='multiple_series',
-            id=5,
         )
-        self.stats.criteria.add(criteria)
-        self.stats.save()
-        self.stats.refresh_from_db()
+        mommy.make('CriteriaToStatsM2M', criteria=criteria, stats=self.stats, use_as='multiple_series', id=5)
         mommy.make('User', date_joined=datetime.datetime(2010, 10, 10, tzinfo=datetime.timezone.utc))
         url = reverse('chart-data', kwargs={'graph_key': 'user_graph'})
         url += "?time_since=2010-10-08"
@@ -446,6 +517,7 @@ class ViewsTests(BaseAuthenticatedClient):
         url += "&select_box_interval=days"
         url += "&select_box_chart_type=discreteBarChart"
         url += "&select_box_multiple_series=5"
+        url += "&debug=True"
         response = self.client.get(url)
         self.assertContains(response, ('"key": "Inactive"'))
         self.assertContains(response, ('"key": "Active"'))
@@ -479,7 +551,8 @@ class AdminToolsStatsModel(TestCase):
         self.assertEqual(self.dashboard_stats_criteria.__str__(), 'call_type')
 
         # DashboardStats model
-        self.dashboard_stats = DashboardStats(
+        self.dashboard_stats = mommy.make(
+            'DashboardStats',
             graph_key='user_graph_test',
             graph_title='User graph',
             model_app_name='auth',
@@ -487,9 +560,7 @@ class AdminToolsStatsModel(TestCase):
             date_field_name='date_joined',
             is_visible=1,
         )
-        self.dashboard_stats.save()
-        self.dashboard_stats.criteria.add(self.dashboard_stats_criteria)
-        self.dashboard_stats.save()
+        mommy.make('CriteriaToStatsM2M', criteria=self.dashboard_stats_criteria, stats=self.dashboard_stats, use_as='multiple_series')
         with self.assertRaises(ValidationError) as e:
             self.dashboard_stats.clean()
         self.assertEqual(e.exception.message_dict, {})
